#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import unittest

from json_environ import Environ
from json_environ.json_environ import DEFAULT_ENV_PATH


class TestJsonEnviron(unittest.TestCase):

    def setUp(self):
        self.env_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "test_env.json")

    def test_default_env_path(self):
        if os.path.exists(DEFAULT_ENV_PATH):
            with open(DEFAULT_ENV_PATH, 'r') as default_env_file:
                env = Environ()
                self.assertEqual(str(env), default_env_file.read())
        else:
            with self.assertRaises(FileNotFoundError):
                env = Environ()

    def test_custom_env_path(self):
        env = Environ(path=self.env_path)
        with open(self.env_path, 'r') as custom_env_file:
            self.assertEqual(str(env), custom_env_file.read())

    def test_existing_key(self):
        env = Environ(path=self.env_path)
        self.assertEqual(env("SECRET"), "secret")
        self.assertEqual(env("SECRET", default=True), "secret")
        self.assertEqual(env("NUMBER"), 3)
        self.assertEqual(env("NUMBER", default=True), 3)
        self.assertEqual(env("DEBUG"), True)
        self.assertEqual(env("WRONG"), False)
        self.assertEqual(env("NULL"), None)
        self.assertEqual(env("LIST"), ["*"])
        self.assertEqual(env("NESTED"), {"NAME": "test", "PASSWORD": 123, "LIST": ["*"], "another": {"ONE": 1.0}})

    def test_not_existing_key_with_default(self):
        env = Environ(path=self.env_path)
        self.assertEqual(env("NOTHING", default=True), True)

    def test_not_existing_key(self):
        env = Environ(path=self.env_path)
        with self.assertRaises(KeyError):
            env("nope")

    def test_existing_nested_key(self):
        env = Environ(path=self.env_path)
        self.assertEqual(env("NESTED:NAME"), "test")
        self.assertEqual(env("NESTED:NAME", default=True), "test")
        self.assertEqual(env("NESTED:PASSWORD"), 123)
        self.assertEqual(env("NESTED:PASSWORD", default=True), 123)
        self.assertEqual(env("NESTED:LIST"), ["*"])
        self.assertEqual(env("NESTED:another:ONE"), 1.0)
        self.assertEqual(env("NESTED:another"), {"ONE": 1.0})

    def test_not_existing_nested_key(self):
        env = Environ(path=self.env_path)
        with self.assertRaises(KeyError):
            env("NESTED:nope")
        with self.assertRaises(KeyError):
            env("nope:nope")

    def test_not_existing_nested_key_with_default(self):
        env = Environ(path=self.env_path)
        self.assertEqual(env("NESTED:nope", default='ok'), 'ok')
        self.assertEqual(env("nope:nope:nope", default='ok'), 'ok')

    def test_custom_separator(self):
        env = Environ(path=self.env_path, key_separator=">")
        with self.assertRaises(KeyError):
            env("NESTED>nope")
        with self.assertRaises(KeyError):
            env("NESTED:NAME")

        self.assertEqual(env("NESTED>nope", default='ok'), 'ok')
        self.assertEqual(env("NESTED>NAME", default=True), "test")
        self.assertEqual(env("NESTED>NAME"), "test")



# -*- coding: utf-8 -*-
import os
import json

DEFAULT_ENV_PATH = "{home}/.env.json".format(home=os.path.expanduser("~"))
DEFAULT_KEY_SEPARATOR = ":"


class NoValue(object):
    pass


class Environ(object):
    __NOTSET = NoValue()

    def __init__(self, path=None, key_separator=DEFAULT_KEY_SEPARATOR):
        self.__path = path if path else DEFAULT_ENV_PATH
        self.key_separator = key_separator

        if not os.path.exists(self.__path):
            raise FileNotFoundError('Is "{}" exist?'.format(self.__path))
        with open(self.__path, 'r') as env_file:
            self.__as_str = env_file.read()
            self._env = json.loads(self.__as_str)

    def __call__(self, key, default=__NOTSET):
        first_key, keys = self._separate_key(key=key)
        try:
            structure = self._env[first_key]
            for nested_key in keys:
                structure = structure[nested_key]
            return structure
        except KeyError:
            if default is self.__NOTSET:
                raise KeyError("{key} is not found in {path}"
                               .format(key=key, path=self.__path))
            return default

    def __str__(self):
        return self.__as_str

    def _separate_key(self, key):
        keys = key.split(sep=self.key_separator)
        first_key = keys.pop(0)
        return first_key, keys
